<?php

namespace BeycanPress\Story\StoryTypes;

use \BeycanPress\Story\Helpers\Response;
use \BeycanPress\Story\Traits\Helpers;

/**
 * Everything about user stories is here
 */
class UserStories
{
    use Helpers;

    /**
     * Register user stories short code
     */
    public function __construct()
    {
        if (is_admin()) {
            // Ajax
            $this->ajaxAction('getStoriesList');
            $this->ajaxAction('createNewStory');
            $this->ajaxAction('deleteMyStory');
        } else {
            add_action('init', function(){
                add_shortcode('bpstory-user-stories', array($this, 'init'));
            });
        }
    }

    public function init()
    {
        if (!is_user_logged_in() && $this->setting('user_stories_only_registered_users_visible')) {
            return esc_html__('Only registered users can see user stories.', 'bpstory');
        } else {

            switch($this->setting('user_stories_story_box_redirect_url')) {
                case 'authorPage':
                    $storyBoxRedirectURL = home_url('author');
                break;
                case 'bbPressProfilePage':
                    $storyBoxRedirectURL = home_url('forums/users');
                break;
                case 'buddyPressProfilePage':
                    $storyBoxRedirectURL = home_url('members');
                break;
                default:
                    $storyBoxRedirectURL = home_url('author');
                break;
            }

            return $this->view('bpstory-js', [
                'id' => 'user-stories',
                'options' => [
                    'creator' => true,
                    "getStoriesParameter" => "getStoriesList",
                    "storyBoxRedirectURL" => $storyBoxRedirectURL
                ]
            ]);
        }
    }

    private function getStoryBoxesQuery()
    {
        return "SELECT 
                    u.user_login, 
                    us.user_id, 
                    us.created_at, 
                    us.story_id, 
                    us.media_url, 
                    us.external_url
            FROM `{$this->db->users}` AS u 
            LEFT JOIN `{$this->db->pluginTables->us->tableName}` AS us ON u.ID = us.user_id
            WHERE us.status = 1
            AND us.created_at >= now() - INTERVAL 1 DAY
            ORDER BY us.created_at ASC";
    }

    private function getStoryBoxesQueryFilterByBuddyPressFirends()
    {
        global $bp;
        return "SELECT 
                    u.user_login, 
                    us.user_id, 
                    us.created_at, 
                    us.story_id, 
                    us.media_url, 
                    us.external_url
            FROM `{$this->db->users}` AS u 
            LEFT JOIN `{$this->db->pluginTables->us->tableName}` AS us ON u.ID = us.user_id
            INNER JOIN `{$bp->friends->table_name}` AS bpf ON bpf.friend_user_id IN (us.user_id)
            WHERE us.status = 1
            AND us.created_at >= now() - INTERVAL 1 DAY 
            AND bpf.is_confirmed = 1
            ORDER BY us.created_at ASC";
    }

    protected function getStoryBoxes()
    {
        $query = $this->getStoryBoxesQuery();

        $storyBoxes = $this->db->get_results($query);

        return $this->decorateStoryBoxes($storyBoxes);
    }

    private function onlyBuddyPressFriendsFilter(array $storyBoxes)
    {
        global $bp;
        $currentUserId = get_current_user_id();

        $query = "SELECT 
            bpf.friend_user_id,
            bpf.initiator_user_id 
            FROM `{$bp->friends->table_name}` AS bpf
            WHERE bpf.is_confirmed = 1
            AND bpf.initiator_user_id = $currentUserId 
            OR bpf.friend_user_id = $currentUserId";

        $friends = $this->db->get_results($query);

        $friends = array_merge(
            array_column($friends, 'initiator_user_id'),
            array_column($friends, 'friend_user_id')
        );
        
        $friends[] = $currentUserId;

        $friends = array_unique($friends);

        return array_filter($storyBoxes, function($story) use ($friends) { 
            return in_array($story->user_id, $friends);
        });
    }

    private function decorateStoryBoxes(array $storyBoxes)
    {
        $newStoryBoxes = [];
        $currentUser = wp_get_current_user();

        if ($this->setting('only_buddypress_friends') && is_user_logged_in() && class_exists('BuddyPress')) {
            $storyBoxes = $this->onlyBuddyPressFriendsFilter($storyBoxes);
        }

        array_map(function($story) use (&$newStoryBoxes, $currentUser) {
            
            $newStoryBoxes[$story->user_id]['storyBox'] = $story->user_login;
            $newStoryBoxes[$story->user_id]['storyBoxImage'] = get_avatar_url($story->user_id);
            $newStoryBoxes[$story->user_id]['lastStoryCreatedAt'] = strtotime($story->created_at);
            $newStoryBoxes[$story->user_id]['lastStoryPublishTime'] = $this->dateToTimeAgo($story->created_at);
            
            // is current user
            if ($currentUser->ID && $currentUser->ID == $story->user_id) {
                $newStoryBoxes[$story->user_id]['currentUser'] = true;
            }

            $newStoryBoxes[$story->user_id]['storyItems'][] = $this->decorateStoryItem($story);

        }, $storyBoxes);

        if ($currentUser->ID && !isset($newStoryBoxes[$currentUser->ID])) {
            $newStoryBoxes[$currentUser->ID]['storyBox'] = $currentUser->user_login;
            $newStoryBoxes[$currentUser->ID]['storyBoxImage'] = get_avatar_url($currentUser->ID);
            $newStoryBoxes[$currentUser->ID]['lastStoryCreatedAt'] = null;
            $newStoryBoxes[$currentUser->ID]['lastStoryPublishTime'] = null;
            $newStoryBoxes[$currentUser->ID]['currentUser'] = true;
            $newStoryBoxes[$currentUser->ID]['storyItems'] = [];
        }

        usort($newStoryBoxes, function($a, $b) {
            return $a['lastStoryCreatedAt'] < $b['lastStoryCreatedAt'];
        });

        $index = array_search($currentUser->user_login, array_column($newStoryBoxes, 'storyBox'));
        if (isset($newStoryBoxes[$index])) {
            $currentUserStoryBox = $newStoryBoxes[$index];
            unset($newStoryBoxes[$index]);
            array_unshift($newStoryBoxes, $currentUserStoryBox);
        }

        return array_values($newStoryBoxes);
    }

    private function decorateStoryItem(object $story)
    {   
        return [
            'storyId' => absint($story->story_id),
            'mediaURL' => $story->media_url,
            'externalURL' => $story->external_url,
            'createdAt' => $story->created_at,
            'publishTime' => $this->dateToTimeAgo($story->created_at)
        ];
    }

    public function getStoriesList()
    {
        if (!is_user_logged_in() && $this->setting('user_stories_only_registered_users_visible')) {
            Response::error();
        }

        $storyBoxes = $this->getStoryBoxes();
        if (!is_user_logged_in() && count($storyBoxes) == 0) {
            Response::success(esc_html__('No story found.', 'bpstory'));
        }

        Response::success(null, $storyBoxes);
    }

    public function createNewStory()
    {
        $media = isset($_POST['media']) ? sanitize_text_field($_POST['media']) : null;
        $externalURL = isset($_POST['externalURL']) ? esc_url($_POST['externalURL']) : null;

        if (!is_user_logged_in()) {
            Response::error(esc_html__('Please login!', 'bpstory'));
        }

        if (is_null($media)) {
            Response::error(esc_html__('Media element not found in request!', 'bpstory'));
        }

        $result = $this->fileAudit($media);
        if (!$result->success) {
            Response::error($result->message);
        }
        
        $uploadedFile = $this->base64FileUpload($media);

        $result = $this->db->pluginTables->us->insert([
            'user_id' => get_current_user_id(),
            'media_url' => $uploadedFile->URL,
            'external_url' => $externalURL,
            'status' => 1 
        ]);

        if (!$result) {
            Response::error(esc_html__('Something went wrong while creating your story.', 'bpstory'));
        } else {
            Response::success(esc_html__('Your story has been successfully published.', 'bpstory'));
        }
        
    }

    public function deleteMyStory()
    {
        $storyId = isset($_POST['storyId']) ? absint($_POST['storyId']) : null;
        
        if (is_null($storyId) || !is_user_logged_in()) {
            Response::error(esc_html__('Bade Request', 'bpstory'));
        }

        $result = $this->db->pluginTables->us->update(['status' => 0], [
            'story_id' => $storyId,
            'user_id' => get_current_user_id(),
            'status' => 1
        ]);
        
        if ($result) {
            Response::success();
        } else {
            Response::error(esc_html__('You do not have a story with this ID!', 'bpstory'));
        }

        Response::success();
    }

}